#!/usr/bin/env python

import os

from nose.tools import eq_, raises

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def teardown():
    index = '../data/sqlite/world.sqlite.index'
    if os.path.exists(index):
        os.unlink(index)

if 'sqlite' in mapnik.DatasourceCache.plugin_names():

    def test_attachdb_with_relative_file():
        # The point table and index is in the qgis_spatiallite.sqlite
        # database.  If either is not found, then this fails
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='point',
                           attachdb='scratch@qgis_spatiallite.sqlite'
                           )
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['pkuid'], 1)

    test_attachdb_with_relative_file.requires_data = True

    def test_attachdb_with_multiple_files():
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='attachedtest',
                           attachdb='scratch1@:memory:,scratch2@:memory:',
                           initdb='''
                create table scratch1.attachedtest (the_geom);
                create virtual table scratch2.idx_attachedtest_the_geom using rtree(pkid,xmin,xmax,ymin,ymax);
                insert into scratch2.idx_attachedtest_the_geom values (1,-7799225.5,-7778571.0,1393264.125,1417719.375);
                '''
                           )
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        # the above should not throw but will result in no features
        eq_(feature, None)

    test_attachdb_with_multiple_files.requires_data = True

    def test_attachdb_with_absolute_file():
        # The point table and index is in the qgis_spatiallite.sqlite
        # database.  If either is not found, then this fails
        ds = mapnik.SQLite(file=os.getcwd() + '/../data/sqlite/world.sqlite',
                           table='point',
                           attachdb='scratch@qgis_spatiallite.sqlite'
                           )
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['pkuid'], 1)

    test_attachdb_with_absolute_file.requires_data = True

    def test_attachdb_with_index():
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='attachedtest',
                           attachdb='scratch@:memory:',
                           initdb='''
                create table scratch.attachedtest (the_geom);
                create virtual table scratch.idx_attachedtest_the_geom using rtree(pkid,xmin,xmax,ymin,ymax);
                insert into scratch.idx_attachedtest_the_geom values (1,-7799225.5,-7778571.0,1393264.125,1417719.375);
                '''
                           )

        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)

    test_attachdb_with_index.requires_data = True

    def test_attachdb_with_explicit_index():
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='attachedtest',
                           index_table='myindex',
                           attachdb='scratch@:memory:',
                           initdb='''
                create table scratch.attachedtest (the_geom);
                create virtual table scratch.myindex using rtree(pkid,xmin,xmax,ymin,ymax);
                insert into scratch.myindex values (1,-7799225.5,-7778571.0,1393264.125,1417719.375);
                '''
                           )
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)

    test_attachdb_with_explicit_index.requires_data = True

    def test_attachdb_with_sql_join():
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select * from world_merc INNER JOIN business on world_merc.iso3 = business.ISO3 limit 100)',
                           attachdb='busines@business.sqlite'
                           )
        eq_(len(ds.fields()), 29)
        eq_(ds.fields(),
            ['OGC_FID',
             'fips',
             'iso2',
             'iso3',
             'un',
             'name',
             'area',
             'pop2005',
             'region',
             'subregion',
             'lon',
             'lat',
             'ISO3:1',
             '1995',
             '1996',
             '1997',
             '1998',
             '1999',
             '2000',
             '2001',
             '2002',
             '2003',
             '2004',
             '2005',
             '2006',
             '2007',
             '2008',
             '2009',
             '2010'])
        eq_(ds.field_types(),
            ['int',
             'str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float',
             'str',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int'])
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature.id(), 1)
        expected = {
            1995: 0,
            1996: 0,
            1997: 0,
            1998: 0,
            1999: 0,
            2000: 0,
            2001: 0,
            2002: 0,
            2003: 0,
            2004: 0,
            2005: 0,
            2006: 0,
            2007: 0,
            2008: 0,
            2009: 0,
            2010: 0,
            # this appears to be sqlites way of
            # automatically handling clashing column names
            'ISO3:1': 'ATG',
            'OGC_FID': 1,
            'area': 44,
            'fips': u'AC',
            'iso2': u'AG',
            'iso3': u'ATG',
            'lat': 17.078,
            'lon': -61.783,
            'name': u'Antigua and Barbuda',
            'pop2005': 83039,
            'region': 19,
            'subregion': 29,
            'un': 28
        }
        for k, v in expected.items():
            try:
                eq_(feature[str(k)], v)
            except:
                #import pdb;pdb.set_trace()
                print('invalid key/v %s/%s for: %s' % (k, v, feature))

    test_attachdb_with_sql_join.requires_data = True

    def test_attachdb_with_sql_join_count():
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select * from world_merc INNER JOIN business on world_merc.iso3 = business.ISO3 limit 100)',
                           attachdb='busines@business.sqlite'
                           )
        eq_(len(ds.fields()), 29)
        eq_(ds.fields(),
            ['OGC_FID',
             'fips',
             'iso2',
             'iso3',
             'un',
             'name',
             'area',
             'pop2005',
             'region',
             'subregion',
             'lon',
             'lat',
             'ISO3:1',
             '1995',
             '1996',
             '1997',
             '1998',
             '1999',
             '2000',
             '2001',
             '2002',
             '2003',
             '2004',
             '2005',
             '2006',
             '2007',
             '2008',
             '2009',
             '2010'])
        eq_(ds.field_types(),
            ['int',
             'str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float',
             'str',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int'])
        eq_(len(list(ds.all_features())), 100)

    test_attachdb_with_sql_join_count.requires_data = True

    def test_attachdb_with_sql_join_count2():
        '''
        sqlite3 world.sqlite
        attach database 'business.sqlite' as business;
        select count(*) from world_merc INNER JOIN business on world_merc.iso3 = business.ISO3;
        '''
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select * from world_merc INNER JOIN business on world_merc.iso3 = business.ISO3)',
                           attachdb='busines@business.sqlite'
                           )
        eq_(len(ds.fields()), 29)
        eq_(ds.fields(),
            ['OGC_FID',
             'fips',
             'iso2',
             'iso3',
             'un',
             'name',
             'area',
             'pop2005',
             'region',
             'subregion',
             'lon',
             'lat',
             'ISO3:1',
             '1995',
             '1996',
             '1997',
             '1998',
             '1999',
             '2000',
             '2001',
             '2002',
             '2003',
             '2004',
             '2005',
             '2006',
             '2007',
             '2008',
             '2009',
             '2010'])
        eq_(ds.field_types(),
            ['int',
             'str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float',
             'str',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int'])
        eq_(len(list(ds.all_features())), 192)

    test_attachdb_with_sql_join_count2.requires_data = True

    def test_attachdb_with_sql_join_count3():
        '''
        select count(*) from (select * from world_merc where 1=1) as world_merc INNER JOIN business on world_merc.iso3 = business.ISO3;
        '''
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select * from (select * from world_merc where !intersects!) as world_merc INNER JOIN business on world_merc.iso3 = business.ISO3)',
                           attachdb='busines@business.sqlite'
                           )
        eq_(len(ds.fields()), 29)
        eq_(ds.fields(),
            ['OGC_FID',
             'fips',
             'iso2',
             'iso3',
             'un',
             'name',
             'area',
             'pop2005',
             'region',
             'subregion',
             'lon',
             'lat',
             'ISO3:1',
             '1995',
             '1996',
             '1997',
             '1998',
             '1999',
             '2000',
             '2001',
             '2002',
             '2003',
             '2004',
             '2005',
             '2006',
             '2007',
             '2008',
             '2009',
             '2010'])
        eq_(ds.field_types(),
            ['int',
             'str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float',
             'str',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int'])
        eq_(len(list(ds.all_features())), 192)

    test_attachdb_with_sql_join_count3.requires_data = True

    def test_attachdb_with_sql_join_count4():
        '''
        select count(*) from (select * from world_merc where 1=1) as world_merc INNER JOIN business on world_merc.iso3 = business.ISO3;
        '''
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select * from (select * from world_merc where !intersects! limit 1) as world_merc INNER JOIN business on world_merc.iso3 = business.ISO3)',
                           attachdb='busines@business.sqlite'
                           )
        eq_(len(ds.fields()), 29)
        eq_(ds.fields(),
            ['OGC_FID',
             'fips',
             'iso2',
             'iso3',
             'un',
             'name',
             'area',
             'pop2005',
             'region',
             'subregion',
             'lon',
             'lat',
             'ISO3:1',
             '1995',
             '1996',
             '1997',
             '1998',
             '1999',
             '2000',
             '2001',
             '2002',
             '2003',
             '2004',
             '2005',
             '2006',
             '2007',
             '2008',
             '2009',
             '2010'])
        eq_(ds.field_types(),
            ['int',
             'str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float',
             'str',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int',
             'int'])
        eq_(len(list(ds.all_features())), 1)

    test_attachdb_with_sql_join_count4.requires_data = True

    def test_attachdb_with_sql_join_count5():
        '''
        select count(*) from (select * from world_merc where 1=1) as world_merc INNER JOIN business on world_merc.iso3 = business.ISO3;
        '''
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select * from (select * from world_merc where !intersects! and 1=2) as world_merc INNER JOIN business on world_merc.iso3 = business.ISO3)',
                           attachdb='busines@business.sqlite'
                           )
        # nothing is able to join to business so we don't pick up business
        # schema
        eq_(len(ds.fields()), 12)
        eq_(ds.fields(),
            ['OGC_FID',
             'fips',
             'iso2',
             'iso3',
             'un',
             'name',
             'area',
             'pop2005',
             'region',
             'subregion',
             'lon',
             'lat'])
        eq_(ds.field_types(),
            ['int',
             'str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float'])
        eq_(len(list(ds.all_features())), 0)

    test_attachdb_with_sql_join_count5.requires_data = True

    def test_subqueries():
        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='world_merc',
                           )
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['OGC_FID'], 1)
        eq_(feature['fips'], u'AC')
        eq_(feature['iso2'], u'AG')
        eq_(feature['iso3'], u'ATG')
        eq_(feature['un'], 28)
        eq_(feature['name'], u'Antigua and Barbuda')
        eq_(feature['area'], 44)
        eq_(feature['pop2005'], 83039)
        eq_(feature['region'], 19)
        eq_(feature['subregion'], 29)
        eq_(feature['lon'], -61.783)
        eq_(feature['lat'], 17.078)

        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select * from world_merc)',
                           )
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['OGC_FID'], 1)
        eq_(feature['fips'], u'AC')
        eq_(feature['iso2'], u'AG')
        eq_(feature['iso3'], u'ATG')
        eq_(feature['un'], 28)
        eq_(feature['name'], u'Antigua and Barbuda')
        eq_(feature['area'], 44)
        eq_(feature['pop2005'], 83039)
        eq_(feature['region'], 19)
        eq_(feature['subregion'], 29)
        eq_(feature['lon'], -61.783)
        eq_(feature['lat'], 17.078)

        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select OGC_FID,GEOMETRY from world_merc)',
                           )
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['OGC_FID'], 1)
        eq_(len(feature), 1)

        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select GEOMETRY,OGC_FID,fips from world_merc)',
                           )
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['OGC_FID'], 1)
        eq_(feature['fips'], u'AC')

        # same as above, except with alias like postgres requires
        # TODO - should we try to make this work?
        # ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
        #    table='(select GEOMETRY,rowid as aliased_id,fips from world_merc) as table',
        #    key_field='aliased_id'
        #    )
        #fs = ds.featureset()
        #feature = fs.next()
        # eq_(feature['aliased_id'],1)
        # eq_(feature['fips'],u'AC')

        ds = mapnik.SQLite(file='../data/sqlite/world.sqlite',
                           table='(select GEOMETRY,OGC_FID,OGC_FID as rowid,fips from world_merc)',
                           )
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['rowid'], 1)
        eq_(feature['fips'], u'AC')

    test_subqueries.requires_data = True

    def test_empty_db():
        ds = mapnik.SQLite(file='../data/sqlite/empty.db',
                           table='empty',
                           )
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)

    test_empty_db.requires_data = True

    @raises(RuntimeError)
    def test_that_nonexistant_query_field_throws(**kwargs):
        ds = mapnik.SQLite(file='../data/sqlite/empty.db',
                           table='empty',
                           )
        eq_(len(ds.fields()), 25)
        eq_(ds.fields(),
            ['OGC_FID',
             'scalerank',
             'labelrank',
             'featurecla',
             'sovereignt',
             'sov_a3',
             'adm0_dif',
             'level',
             'type',
             'admin',
             'adm0_a3',
             'geou_dif',
             'name',
             'abbrev',
             'postal',
             'name_forma',
             'terr_',
             'name_sort',
             'map_color',
             'pop_est',
             'gdp_md_est',
             'fips_10_',
             'iso_a2',
             'iso_a3',
             'iso_n3'])
        eq_(ds.field_types(),
            ['int',
             'int',
             'int',
             'str',
             'str',
             'str',
             'float',
             'float',
             'str',
             'str',
             'str',
             'float',
             'str',
             'str',
             'str',
             'str',
             'str',
             'str',
             'float',
             'float',
             'float',
             'float',
             'str',
             'str',
             'float'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        # also add an invalid one, triggering throw
        query.add_property_name('bogus')
        ds.features(query)

    test_that_nonexistant_query_field_throws.requires_data = True

    def test_intersects_token1():
        ds = mapnik.SQLite(file='../data/sqlite/empty.db',
                           table='(select * from empty where !intersects!)',
                           )
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)

    test_intersects_token1.requires_data = True

    def test_intersects_token2():
        ds = mapnik.SQLite(file='../data/sqlite/empty.db',
                           table='(select * from empty where "a"!="b" and !intersects!)',
                           )
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)

    test_intersects_token2.requires_data = True

    def test_intersects_token3():
        ds = mapnik.SQLite(file='../data/sqlite/empty.db',
                           table='(select * from empty where "a"!="b" and !intersects!)',
                           )
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)

    test_intersects_token3.requires_data = True

    # https://github.com/mapnik/mapnik/issues/1537
    # this works because key_field is manually set
    def test_db_with_one_text_column():
        # form up an in-memory test db
        wkb = '010100000000000000000000000000000000000000'
        ds = mapnik.SQLite(file=':memory:',
                           table='test1',
                           initdb='''
                create table test1 (alias TEXT,geometry BLOB);
                insert into test1 values ("test",x'%s');
                ''' % wkb,
                           extent='-180,-60,180,60',
                           use_spatial_index=False,
                           key_field='alias'
                           )
        eq_(len(ds.fields()), 1)
        eq_(ds.fields(), ['alias'])
        eq_(ds.field_types(), ['str'])
        fs = list(ds.all_features())
        eq_(len(fs), 1)
        feat = fs[0]
        eq_(feat.id(), 0)  # should be 1?
        eq_(feat['alias'], 'test')
        eq_(feat.geometry.to_wkt(), 'POINT(0 0)')

    def test_db_with_one_untyped_column():
        # form up an in-memory test db
        wkb = '010100000000000000000000000000000000000000'
        ds = mapnik.SQLite(file=':memory:',
                           table='test1',
                           initdb='''
                create table test1 (geometry BLOB, untyped);
                insert into test1 values (x'%s', 'untyped');
            ''' % wkb,
                           extent='-180,-60,180,60',
                           use_spatial_index=False,
                           key_field='rowid'
                           )

        # ensure the untyped column is found
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['rowid', 'untyped'])
        eq_(ds.field_types(), ['int', 'str'])

    def test_db_with_one_untyped_column_using_subquery():
        # form up an in-memory test db
        wkb = '010100000000000000000000000000000000000000'
        ds = mapnik.SQLite(file=':memory:',
                           table='(SELECT rowid, geometry, untyped FROM test1)',
                           initdb='''
                create table test1 (geometry BLOB, untyped);
                insert into test1 values (x'%s', 'untyped');
            ''' % wkb,
                           extent='-180,-60,180,60',
                           use_spatial_index=False,
                           key_field='rowid'
                           )

        # ensure the untyped column is found
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['rowid', 'untyped', 'rowid'])
        eq_(ds.field_types(), ['int', 'str', 'int'])

    def test_that_64bit_int_fields_work():
        ds = mapnik.SQLite(file='../data/sqlite/64bit_int.sqlite',
                           table='int_table',
                           use_spatial_index=False
                           )
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['OGC_FID', 'id', 'bigint'])
        eq_(ds.field_types(), ['int', 'int', 'int'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat.id(), 1)
        eq_(feat['OGC_FID'], 1)
        eq_(feat['bigint'], 2147483648)
        feat = fs.next()
        eq_(feat.id(), 2)
        eq_(feat['OGC_FID'], 2)
        eq_(feat['bigint'], 922337203685477580)

    test_that_64bit_int_fields_work.requires_data = True

    def test_null_id_field():
        # silence null key warning:
        # https://github.com/mapnik/mapnik/issues/1889
        default_logging_severity = mapnik.logger.get_severity()
        mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
        # form up an in-memory test db
        wkb = '010100000000000000000000000000000000000000'
        # note: the osm_id should be declared INTEGER PRIMARY KEY
        # but in this case we intentionally do not make this a valid pkey
        # otherwise sqlite would turn the null into a valid, serial id
        ds = mapnik.SQLite(file=':memory:',
                           table='test1',
                           initdb='''
                create table test1 (osm_id INTEGER,geometry BLOB);
                insert into test1 values (null,x'%s');
                ''' % wkb,
                           extent='-180,-60,180,60',
                           use_spatial_index=False,
                           key_field='osm_id'
                           )
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)
        mapnik.logger.set_severity(default_logging_severity)

if __name__ == "__main__":
    setup()
    result = run_all(eval(x) for x in dir() if x.startswith("test_"))
    teardown()
    exit(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_image_16_8_simple():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray16)
    im.set_pixel(0, 0, 256)
    im.set_pixel(0, 1, 999)
    im.set_pixel(1, 0, 5)
    im.set_pixel(1, 1, 2)
    im2 = im.copy(mapnik.ImageType.gray8)
    eq_(im2.get_pixel(0, 0), 255)
    eq_(im2.get_pixel(0, 1), 255)
    eq_(im2.get_pixel(1, 0), 5)
    eq_(im2.get_pixel(1, 1), 2)
    # Cast back!
    im = im2.copy(mapnik.ImageType.gray16)
    eq_(im.get_pixel(0, 0), 255)
    eq_(im.get_pixel(0, 1), 255)
    eq_(im.get_pixel(1, 0), 5)
    eq_(im.get_pixel(1, 1), 2)


def test_image_32f_8_simple():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray32f)
    im.set_pixel(0, 0, 120.1234)
    im.set_pixel(0, 1, -23.4)
    im.set_pixel(1, 0, 120.6)
    im.set_pixel(1, 1, 360.2)
    im2 = im.copy(mapnik.ImageType.gray8)
    eq_(im2.get_pixel(0, 0), 120)
    eq_(im2.get_pixel(0, 1), 0)
    eq_(im2.get_pixel(1, 0), 120)  # Notice this is truncated!
    eq_(im2.get_pixel(1, 1), 255)


def test_image_offset_and_scale():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray16)
    eq_(im.offset, 0.0)
    eq_(im.scaling, 1.0)
    im.offset = 1.0
    im.scaling = 2.0
    eq_(im.offset, 1.0)
    eq_(im.scaling, 2.0)


def test_image_16_8_scale_and_offset():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray16)
    im.set_pixel(0, 0, 256)
    im.set_pixel(0, 1, 258)
    im.set_pixel(1, 0, 99999)
    im.set_pixel(1, 1, 615)
    offset = 255
    scaling = 3
    im2 = im.copy(mapnik.ImageType.gray8, offset, scaling)
    eq_(im2.get_pixel(0, 0), 0)
    eq_(im2.get_pixel(0, 1), 1)
    eq_(im2.get_pixel(1, 0), 255)
    eq_(im2.get_pixel(1, 1), 120)
    # pixels will be a little off due to offsets in reverting!
    im3 = im2.copy(mapnik.ImageType.gray16)
    eq_(im3.get_pixel(0, 0), 255)  # Rounding error with ints
    eq_(im3.get_pixel(0, 1), 258)  # same
    # The other one was way out of range for our scale/offset
    eq_(im3.get_pixel(1, 0), 1020)
    eq_(im3.get_pixel(1, 1), 615)  # same


def test_image_16_32f_scale_and_offset():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray16)
    im.set_pixel(0, 0, 256)
    im.set_pixel(0, 1, 258)
    im.set_pixel(1, 0, 0)
    im.set_pixel(1, 1, 615)
    offset = 255
    scaling = 3.2
    im2 = im.copy(mapnik.ImageType.gray32f, offset, scaling)
    eq_(im2.get_pixel(0, 0), 0.3125)
    eq_(im2.get_pixel(0, 1), 0.9375)
    eq_(im2.get_pixel(1, 0), -79.6875)
    eq_(im2.get_pixel(1, 1), 112.5)
    im3 = im2.copy(mapnik.ImageType.gray16)
    eq_(im3.get_pixel(0, 0), 256)
    eq_(im3.get_pixel(0, 1), 258)
    eq_(im3.get_pixel(1, 0), 0)
    eq_(im3.get_pixel(1, 1), 615)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
import atexit
import os
import sys
import threading
from subprocess import PIPE, Popen

from nose.tools import eq_, raises

import mapnik

from .utilities import execution_path, run_all

PYTHON3 = sys.version_info[0] == 3
if PYTHON3:
    long = int


MAPNIK_TEST_DBNAME = 'mapnik-tmp-postgis-test-db'
POSTGIS_TEMPLATE_DBNAME = 'template_postgis'
SHAPEFILE = os.path.join(execution_path('.'), '../data/shp/world_merc.shp')


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def call(cmd, silent=False):
    stdin, stderr = Popen(cmd, shell=True, stdout=PIPE,
                          stderr=PIPE).communicate()
    if not stderr:
        return stdin.strip()
    msg = str(stderr).lower()
    if not silent and 'error' in msg\
       or 'not found' in msg or 'not recognized as an internal' in msg\
       or 'bad connection' in msg or 'could not connect' in msg:
        raise RuntimeError(msg.strip())


def psql_can_connect():
    """Test ability to connect to a postgis template db with no options.

    Basically, to run these tests your user must have full read
    access over unix sockets without supplying a password. This
    keeps these tests simple and focused on postgis not on postgres
    auth issues.
    """
    try:
        call('psql %s -c "select postgis_version()"' % POSTGIS_TEMPLATE_DBNAME)
        return True
    except RuntimeError:
        print('Notice: skipping postgis tests (connection)')
        return False


def shp2pgsql_on_path():
    """Test for presence of shp2pgsql on the user path.

    We require this program to load test data into a temporarily database.
    """
    try:
        call('shp2pgsql')
        return True
    except RuntimeError:
        print('Notice: skipping postgis tests (shp2pgsql)')
        return False


def createdb_and_dropdb_on_path():
    """Test for presence of dropdb/createdb on user path.

    We require these programs to setup and teardown the testing db.
    """
    try:
        call('createdb --help')
        call('dropdb --help')
        return True
    except RuntimeError:
        print('Notice: skipping postgis tests (createdb/dropdb)')
        return False

insert_table_1 = """
CREATE TABLE test(gid serial PRIMARY KEY, geom geometry);
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;POINT(-2 2)'));
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;MULTIPOINT(2 1,1 2)'));
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;LINESTRING(0 0,1 1,1 2)'));
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;MULTILINESTRING((1 0,0 1,3 2),(3 2,5 4))'));
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;POLYGON((0 0,4 0,4 4,0 4,0 0),(1 1, 2 1, 2 2, 1 2,1 1))'));
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;MULTIPOLYGON(((1 1,3 1,3 3,1 3,1 1),(1 1,2 1,2 2,1 2,1 1)), ((-1 -1,-1 -2,-2 -2,-2 -1,-1 -1)))'));
INSERT INTO test(geom) values (GeomFromEWKT('SRID=4326;GEOMETRYCOLLECTION(POLYGON((1 1, 2 1, 2 2, 1 2,1 1)),POINT(2 3),LINESTRING(2 3,3 4))'));
"""

insert_table_2 = """
CREATE TABLE test2(manual_id int4 PRIMARY KEY, geom geometry);
INSERT INTO test2(manual_id, geom) values (0, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test2(manual_id, geom) values (1, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test2(manual_id, geom) values (1000, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test2(manual_id, geom) values (-1000, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test2(manual_id, geom) values (2147483647, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test2(manual_id, geom) values (-2147483648, GeomFromEWKT('SRID=4326;POINT(0 0)'));
"""

insert_table_3 = """
CREATE TABLE test3(non_id bigint, manual_id int4, geom geometry);
INSERT INTO test3(non_id, manual_id, geom) values (9223372036854775807, 0, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test3(non_id, manual_id, geom) values (9223372036854775807, 1, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test3(non_id, manual_id, geom) values (9223372036854775807, 1000, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test3(non_id, manual_id, geom) values (9223372036854775807, -1000, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test3(non_id, manual_id, geom) values (9223372036854775807, 2147483647, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test3(non_id, manual_id, geom) values (9223372036854775807, -2147483648, GeomFromEWKT('SRID=4326;POINT(0 0)'));
"""

insert_table_4 = """
CREATE TABLE test4(non_id int4, manual_id int8 PRIMARY KEY, geom geometry);
INSERT INTO test4(non_id, manual_id, geom) values (0, 0, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test4(non_id, manual_id, geom) values (0, 1, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test4(non_id, manual_id, geom) values (0, 1000, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test4(non_id, manual_id, geom) values (0, -1000, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test4(non_id, manual_id, geom) values (0, 2147483647, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test4(non_id, manual_id, geom) values (0, -2147483648, GeomFromEWKT('SRID=4326;POINT(0 0)'));
"""

insert_table_5 = """
CREATE TABLE test5(non_id int4, manual_id numeric PRIMARY KEY, geom geometry);
INSERT INTO test5(non_id, manual_id, geom) values (0, -1, GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test5(non_id, manual_id, geom) values (0, 1, GeomFromEWKT('SRID=4326;POINT(0 0)'));
"""

insert_table_5b = '''
CREATE TABLE "tableWithMixedCase"(gid serial PRIMARY KEY, geom geometry);
INSERT INTO "tableWithMixedCase"(geom) values (ST_MakePoint(0,0));
INSERT INTO "tableWithMixedCase"(geom) values (ST_MakePoint(0,1));
INSERT INTO "tableWithMixedCase"(geom) values (ST_MakePoint(1,0));
INSERT INTO "tableWithMixedCase"(geom) values (ST_MakePoint(1,1));
'''

insert_table_6 = '''
CREATE TABLE test6(first_id int4, second_id int4,PRIMARY KEY (first_id,second_id), geom geometry);
INSERT INTO test6(first_id, second_id, geom) values (0, 0, GeomFromEWKT('SRID=4326;POINT(0 0)'));
'''

insert_table_7 = '''
CREATE TABLE test7(gid serial PRIMARY KEY, geom geometry);
INSERT INTO test7(gid, geom) values (1, GeomFromEWKT('SRID=4326;GEOMETRYCOLLECTION(MULTILINESTRING((10 10,20 20,10 40),(40 40,30 30,40 20,30 10)),LINESTRING EMPTY)'));
'''

insert_table_8 = '''
CREATE TABLE test8(gid serial PRIMARY KEY,int_field bigint, geom geometry);
INSERT INTO test8(gid, int_field, geom) values (1, 2147483648, ST_MakePoint(1,1));
INSERT INTO test8(gid, int_field, geom) values (2, 922337203685477580, ST_MakePoint(1,1));
'''

insert_table_9 = '''
CREATE TABLE test9(gid serial PRIMARY KEY, name varchar, geom geometry);
INSERT INTO test9(gid, name, geom) values (1, 'name', ST_MakePoint(1,1));
INSERT INTO test9(gid, name, geom) values (2, '', ST_MakePoint(1,1));
INSERT INTO test9(gid, name, geom) values (3, null, ST_MakePoint(1,1));
'''

insert_table_10 = '''
CREATE TABLE test10(gid serial PRIMARY KEY, bool_field boolean, geom geometry);
INSERT INTO test10(gid, bool_field, geom) values (1, TRUE, ST_MakePoint(1,1));
INSERT INTO test10(gid, bool_field, geom) values (2, FALSE, ST_MakePoint(1,1));
INSERT INTO test10(gid, bool_field, geom) values (3, null, ST_MakePoint(1,1));
'''

insert_table_11 = """
CREATE TABLE test11(gid serial PRIMARY KEY, label varchar(40), geom geometry);
INSERT INTO test11(label,geom) values ('label_1',GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test11(label,geom) values ('label_2',GeomFromEWKT('SRID=4326;POINT(-2 2)'));
INSERT INTO test11(label,geom) values ('label_3',GeomFromEWKT('SRID=4326;MULTIPOINT(2 1,1 2)'));
INSERT INTO test11(label,geom) values ('label_4',GeomFromEWKT('SRID=4326;LINESTRING(0 0,1 1,1 2)'));
INSERT INTO test11(label,geom) values ('label_5',GeomFromEWKT('SRID=4326;MULTILINESTRING((1 0,0 1,3 2),(3 2,5 4))'));
INSERT INTO test11(label,geom) values ('label_6',GeomFromEWKT('SRID=4326;POLYGON((0 0,4 0,4 4,0 4,0 0),(1 1, 2 1, 2 2, 1 2,1 1))'));
INSERT INTO test11(label,geom) values ('label_7',GeomFromEWKT('SRID=4326;MULTIPOLYGON(((1 1,3 1,3 3,1 3,1 1),(1 1,2 1,2 2,1 2,1 1)), ((-1 -1,-1 -2,-2 -2,-2 -1,-1 -1)))'));
INSERT INTO test11(label,geom) values ('label_8',GeomFromEWKT('SRID=4326;GEOMETRYCOLLECTION(POLYGON((1 1, 2 1, 2 2, 1 2,1 1)),POINT(2 3),LINESTRING(2 3,3 4))'));
"""

insert_table_12 = """
CREATE TABLE test12(gid serial PRIMARY KEY, name varchar(40), geom geometry);
INSERT INTO test12(name,geom) values ('Point',GeomFromEWKT('SRID=4326;POINT(0 0)'));
INSERT INTO test12(name,geom) values ('PointZ',GeomFromEWKT('SRID=4326;POINTZ(0 0 0)'));
INSERT INTO test12(name,geom) values ('PointM',GeomFromEWKT('SRID=4326;POINTM(0 0 0)'));
INSERT INTO test12(name,geom) values ('PointZM',GeomFromEWKT('SRID=4326;POINTZM(0 0 0 0)'));
INSERT INTO test12(name,geom) values ('MultiPoint',GeomFromEWKT('SRID=4326;MULTIPOINT(0 0, 1 1)'));
INSERT INTO test12(name,geom) values ('MultiPointZ',GeomFromEWKT('SRID=4326;MULTIPOINTZ(0 0 0, 1 1 1)'));
INSERT INTO test12(name,geom) values ('MultiPointM',GeomFromEWKT('SRID=4326;MULTIPOINTM(0 0 0, 1 1 1)'));
INSERT INTO test12(name,geom) values ('MultiPointZM',GeomFromEWKT('SRID=4326;MULTIPOINTZM(0 0 0 0, 1 1 1 1)'));
INSERT INTO test12(name,geom) values ('LineString',GeomFromEWKT('SRID=4326;LINESTRING(0 0, 1 1)'));
INSERT INTO test12(name,geom) values ('LineStringZ',GeomFromEWKT('SRID=4326;LINESTRINGZ(0 0 0, 1 1 1)'));
INSERT INTO test12(name,geom) values ('LineStringM',GeomFromEWKT('SRID=4326;LINESTRINGM(0 0 0, 1 1 1)'));
INSERT INTO test12(name,geom) values ('LineStringZM',GeomFromEWKT('SRID=4326;LINESTRINGZM(0 0 0 0, 1 1 1 1)'));
INSERT INTO test12(name,geom) values ('Polygon',GeomFromEWKT('SRID=4326;POLYGON((0 0, 1 1, 2 2, 0 0))'));
INSERT INTO test12(name,geom) values ('PolygonZ',GeomFromEWKT('SRID=4326;POLYGONZ((0 0 0, 1 1 1, 2 2 2, 0 0 0))'));
INSERT INTO test12(name,geom) values ('PolygonM',GeomFromEWKT('SRID=4326;POLYGONZ((0 0 0, 1 1 1, 2 2 2, 0 0 0))'));
INSERT INTO test12(name,geom) values ('PolygonZM',GeomFromEWKT('SRID=4326;POLYGONZM((0 0 0 0, 1 1 1 1, 2 2 2 2, 0 0 0 0))'));
INSERT INTO test12(name,geom) values ('MultiLineString',GeomFromEWKT('SRID=4326;MULTILINESTRING((0 0, 1 1),(2 2, 3 3))'));
INSERT INTO test12(name,geom) values ('MultiLineStringZ',GeomFromEWKT('SRID=4326;MULTILINESTRINGZ((0 0 0, 1 1 1),(2 2 2, 3 3 3))'));
INSERT INTO test12(name,geom) values ('MultiLineStringM',GeomFromEWKT('SRID=4326;MULTILINESTRINGM((0 0 0, 1 1 1),(2 2 2, 3 3 3))'));
INSERT INTO test12(name,geom) values ('MultiLineStringZM',GeomFromEWKT('SRID=4326;MULTILINESTRINGZM((0 0 0 0, 1 1 1 1),(2 2 2 2, 3 3 3 3))'));
INSERT INTO test12(name,geom) values ('MultiPolygon',GeomFromEWKT('SRID=4326;MULTIPOLYGON(((0 0, 1 1, 2 2, 0 0)),((0 0, 1 1, 2 2, 0 0)))'));
INSERT INTO test12(name,geom) values ('MultiPolygonZ',GeomFromEWKT('SRID=4326;MULTIPOLYGONZ(((0 0 0, 1 1 1, 2 2 2, 0 0 0)),((0 0 0, 1 1 1, 2 2 2, 0 0 0)))'));
INSERT INTO test12(name,geom) values ('MultiPolygonM',GeomFromEWKT('SRID=4326;MULTIPOLYGONM(((0 0 0, 1 1 1, 2 2 2, 0 0 0)),((0 0 0, 1 1 1, 2 2 2, 0 0 0)))'));
INSERT INTO test12(name,geom) values ('MultiPolygonZM',GeomFromEWKT('SRID=4326;MULTIPOLYGONZM(((0 0 0 0, 1 1 1 1, 2 2 2 2, 0 0 0 0)),((0 0 0 0, 1 1 1 1, 2 2 2 2, 0 0 0 0)))'));
"""


def postgis_setup():
    call('dropdb %s' % MAPNIK_TEST_DBNAME, silent=True)
    call(
        'createdb -T %s %s' %
        (POSTGIS_TEMPLATE_DBNAME,
         MAPNIK_TEST_DBNAME),
        silent=False)
    call('shp2pgsql -s 3857 -g geom -W LATIN1 %s world_merc | psql -q %s' %
         (SHAPEFILE, MAPNIK_TEST_DBNAME), silent=True)
    call(
        '''psql -q %s -c "CREATE TABLE \"empty\" (key serial);SELECT AddGeometryColumn('','empty','geom','-1','GEOMETRY',4);"''' %
        MAPNIK_TEST_DBNAME,
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_1),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_2),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_3),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_4),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_5),
        silent=False)
    call(
        """psql -q %s -c '%s'""" %
        (MAPNIK_TEST_DBNAME,
         insert_table_5b),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_6),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_7),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_8),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_9),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_10),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_11),
        silent=False)
    call(
        '''psql -q %s -c "%s"''' %
        (MAPNIK_TEST_DBNAME,
         insert_table_12),
        silent=False)


def postgis_takedown():
    pass
    # fails as the db is in use: https://github.com/mapnik/mapnik/issues/960
    #call('dropdb %s' % MAPNIK_TEST_DBNAME)

if 'postgis' in mapnik.DatasourceCache.plugin_names() \
        and createdb_and_dropdb_on_path() \
        and psql_can_connect() \
        and shp2pgsql_on_path():

    # initialize test database
    postgis_setup()

    def test_feature():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='world_merc')
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['gid'], 1)
        eq_(feature['fips'], u'AC')
        eq_(feature['iso2'], u'AG')
        eq_(feature['iso3'], u'ATG')
        eq_(feature['un'], 28)
        eq_(feature['name'], u'Antigua and Barbuda')
        eq_(feature['area'], 44)
        eq_(feature['pop2005'], 83039)
        eq_(feature['region'], 19)
        eq_(feature['subregion'], 29)
        eq_(feature['lon'], -61.783)
        eq_(feature['lat'], 17.078)
        meta = ds.describe()
        eq_(meta['srid'], 3857)
        eq_(meta.get('key_field'), None)
        eq_(meta['encoding'], u'UTF8')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Polygon)

    def test_subquery():
        ds = mapnik.PostGIS(
            dbname=MAPNIK_TEST_DBNAME,
            table='(select * from world_merc) as w')
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['gid'], 1)
        eq_(feature['fips'], u'AC')
        eq_(feature['iso2'], u'AG')
        eq_(feature['iso3'], u'ATG')
        eq_(feature['un'], 28)
        eq_(feature['name'], u'Antigua and Barbuda')
        eq_(feature['area'], 44)
        eq_(feature['pop2005'], 83039)
        eq_(feature['region'], 19)
        eq_(feature['subregion'], 29)
        eq_(feature['lon'], -61.783)
        eq_(feature['lat'], 17.078)
        meta = ds.describe()
        eq_(meta['srid'], 3857)
        eq_(meta.get('key_field'), None)
        eq_(meta['encoding'], u'UTF8')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Polygon)

        ds = mapnik.PostGIS(
            dbname=MAPNIK_TEST_DBNAME,
            table='(select gid,geom,fips as _fips from world_merc) as w')
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['gid'], 1)
        eq_(feature['_fips'], u'AC')
        eq_(len(feature), 2)
        meta = ds.describe()
        eq_(meta['srid'], 3857)
        eq_(meta.get('key_field'), None)
        eq_(meta['encoding'], u'UTF8')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Polygon)

    def test_bad_connection():
        try:
            ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,
                                table='test',
                                max_size=20,
                                geometry_field='geom',
                                user="rolethatdoesnotexist")
        except Exception as e:
            assert 'role "rolethatdoesnotexist" does not exist' in str(e) or \
                'authentication failed for user "rolethatdoesnotexist"' in str(e)

    def test_empty_db():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='empty')
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)
        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), None)
        eq_(meta['encoding'], u'UTF8')
        eq_(meta['geometry_type'], None)

    def test_manual_srid():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, srid=99, table='empty')
        fs = ds.featureset()
        feature = None
        try:
            feature = fs.next()
        except StopIteration:
            pass
        eq_(feature, None)
        meta = ds.describe()
        eq_(meta['srid'], 99)
        eq_(meta.get('key_field'), None)
        eq_(meta['encoding'], u'UTF8')
        eq_(meta['geometry_type'], None)

    def test_geometry_detection():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test',
                            geometry_field='geom')
        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Collection)

        # will fail with postgis 2.0 because it automatically adds a geometry_columns entry
        # ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,table='test',
        #                   geometry_field='geom',
        #                    row_limit=1)
        # eq_(ds.describe()['geometry_type'],mapnik.DataGeometryType.Point)

    @raises(RuntimeError)
    def test_that_nonexistant_query_field_throws(**kwargs):
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='empty')
        eq_(len(ds.fields()), 1)
        eq_(ds.fields(), ['key'])
        eq_(ds.field_types(), ['int'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        # also add an invalid one, triggering throw
        query.add_property_name('bogus')
        ds.features(query)

    def test_auto_detection_of_unique_feature_id_32_bit():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test2',
                            geometry_field='geom',
                            autodetect_key_field=True)
        fs = ds.featureset()
        f = fs.next()
        eq_(len(ds.fields()),len(f.attributes))
        eq_(f['manual_id'], 0)
        eq_(fs.next()['manual_id'], 1)
        eq_(fs.next()['manual_id'], 1000)
        eq_(fs.next()['manual_id'], -1000)
        eq_(fs.next()['manual_id'], 2147483647)
        eq_(fs.next()['manual_id'], -2147483648)

        fs = ds.featureset()
        eq_(fs.next().id(), 0)
        eq_(fs.next().id(), 1)
        eq_(fs.next().id(), 1000)
        eq_(fs.next().id(), -1000)
        eq_(fs.next().id(), 2147483647)
        eq_(fs.next().id(), -2147483648)
        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), u'manual_id')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_auto_detection_of_unique_feature_id_32_bit_no_attribute():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test2',
                            geometry_field='geom',
                            autodetect_key_field=True,
                            key_field_as_attribute=False)
        fs = ds.featureset()
        f = fs.next()
        eq_(len(ds.fields()),len(f.attributes))
        eq_(len(ds.fields()),0)
        eq_(len(f.attributes),0)
        eq_(f.id(), 0)
        eq_(fs.next().id(), 1)
        eq_(fs.next().id(), 1000)
        eq_(fs.next().id(), -1000)
        eq_(fs.next().id(), 2147483647)
        eq_(fs.next().id(), -2147483648)
        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), u'manual_id')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_auto_detection_will_fail_since_no_primary_key():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test3',
                            geometry_field='geom',
                            autodetect_key_field=False)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['manual_id'], 0)
        eq_(feat['non_id'],9223372036854775807)
        eq_(fs.next()['manual_id'], 1)
        eq_(fs.next()['manual_id'], 1000)
        eq_(fs.next()['manual_id'], -1000)
        eq_(fs.next()['manual_id'], 2147483647)
        eq_(fs.next()['manual_id'], -2147483648)

        # since no valid primary key will be detected the fallback
        # is auto-incrementing counter
        fs = ds.featureset()
        eq_(fs.next().id(), 1)
        eq_(fs.next().id(), 2)
        eq_(fs.next().id(), 3)
        eq_(fs.next().id(), 4)
        eq_(fs.next().id(), 5)
        eq_(fs.next().id(), 6)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    @raises(RuntimeError)
    def test_auto_detection_will_fail_and_should_throw():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test3',
                            geometry_field='geom',
                            autodetect_key_field=True)
        ds.featureset()

    def test_auto_detection_of_unique_feature_id_64_bit():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test4',
                            geometry_field='geom',
                            autodetect_key_field=True)
        fs = ds.featureset()
        f = fs.next()
        eq_(len(ds.fields()),len(f.attributes))
        eq_(f['manual_id'], 0)
        eq_(fs.next()['manual_id'], 1)
        eq_(fs.next()['manual_id'], 1000)
        eq_(fs.next()['manual_id'], -1000)
        eq_(fs.next()['manual_id'], 2147483647)
        eq_(fs.next()['manual_id'], -2147483648)

        fs = ds.featureset()
        eq_(fs.next().id(), 0)
        eq_(fs.next().id(), 1)
        eq_(fs.next().id(), 1000)
        eq_(fs.next().id(), -1000)
        eq_(fs.next().id(), 2147483647)
        eq_(fs.next().id(), -2147483648)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), u'manual_id')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_disabled_auto_detection_and_subquery():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''(select geom, 'a'::varchar as name from test2) as t''',
                            geometry_field='geom',
                            autodetect_key_field=False)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat.id(), 1)
        eq_(feat['name'], 'a')
        feat = fs.next()
        eq_(feat.id(), 2)
        eq_(feat['name'], 'a')
        feat = fs.next()
        eq_(feat.id(), 3)
        eq_(feat['name'], 'a')
        feat = fs.next()
        eq_(feat.id(), 4)
        eq_(feat['name'], 'a')
        feat = fs.next()
        eq_(feat.id(), 5)
        eq_(feat['name'], 'a')
        feat = fs.next()
        eq_(feat.id(), 6)
        eq_(feat['name'], 'a')

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_auto_detection_and_subquery_including_key():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''(select geom, manual_id from test2) as t''',
                            geometry_field='geom',
                            autodetect_key_field=True)
        fs = ds.featureset()
        f = fs.next()
        eq_(len(ds.fields()),len(f.attributes))
        eq_(f['manual_id'], 0)
        eq_(fs.next()['manual_id'], 1)
        eq_(fs.next()['manual_id'], 1000)
        eq_(fs.next()['manual_id'], -1000)
        eq_(fs.next()['manual_id'], 2147483647)
        eq_(fs.next()['manual_id'], -2147483648)

        fs = ds.featureset()
        eq_(fs.next().id(), 0)
        eq_(fs.next().id(), 1)
        eq_(fs.next().id(), 1000)
        eq_(fs.next().id(), -1000)
        eq_(fs.next().id(), 2147483647)
        eq_(fs.next().id(), -2147483648)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), u'manual_id')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    @raises(RuntimeError)
    def test_auto_detection_of_invalid_numeric_primary_key():
        mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''(select geom, manual_id::numeric from test2) as t''',
                       geometry_field='geom',
                       autodetect_key_field=True)

    @raises(RuntimeError)
    def test_auto_detection_of_invalid_multiple_keys():
        mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''test6''',
                       geometry_field='geom',
                       autodetect_key_field=True)

    @raises(RuntimeError)
    def test_auto_detection_of_invalid_multiple_keys_subquery():
        mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''(select first_id,second_id,geom from test6) as t''',
                       geometry_field='geom',
                       autodetect_key_field=True)

    def test_manually_specified_feature_id_field():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test4',
                            geometry_field='geom',
                            key_field='manual_id',
                            autodetect_key_field=True)
        fs = ds.featureset()
        f = fs.next()
        eq_(len(ds.fields()),len(f.attributes))
        eq_(f['manual_id'], 0)
        eq_(fs.next()['manual_id'], 1)
        eq_(fs.next()['manual_id'], 1000)
        eq_(fs.next()['manual_id'], -1000)
        eq_(fs.next()['manual_id'], 2147483647)
        eq_(fs.next()['manual_id'], -2147483648)

        fs = ds.featureset()
        eq_(fs.next().id(), 0)
        eq_(fs.next().id(), 1)
        eq_(fs.next().id(), 1000)
        eq_(fs.next().id(), -1000)
        eq_(fs.next().id(), 2147483647)
        eq_(fs.next().id(), -2147483648)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), u'manual_id')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_numeric_type_feature_id_field():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test5',
                            geometry_field='geom',
                            autodetect_key_field=False)
        fs = ds.featureset()
        eq_(fs.next()['manual_id'], -1)
        eq_(fs.next()['manual_id'], 1)

        fs = ds.featureset()
        eq_(fs.next().id(), 1)
        eq_(fs.next().id(), 2)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_querying_table_with_mixed_case():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='"tableWithMixedCase"',
                            geometry_field='geom',
                            autodetect_key_field=True)
        fs = ds.featureset()
        for id in range(1, 5):
            eq_(fs.next().id(), id)

        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), u'gid')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_querying_subquery_with_mixed_case():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='(SeLeCt * FrOm "tableWithMixedCase") as MixedCaseQuery',
                            geometry_field='geom',
                            autodetect_key_field=True)
        fs = ds.featureset()
        for id in range(1, 5):
            eq_(fs.next().id(), id)

        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), u'gid')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_bbox_token_in_subquery1():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''
           (SeLeCt * FrOm "tableWithMixedCase" where geom && !bbox! ) as MixedCaseQuery''',
                            geometry_field='geom',
                            autodetect_key_field=True)
        fs = ds.featureset()
        for id in range(1, 5):
            eq_(fs.next().id(), id)

        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), u'gid')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_bbox_token_in_subquery2():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''
           (SeLeCt * FrOm "tableWithMixedCase" where ST_Intersects(geom,!bbox!) ) as MixedCaseQuery''',
                            geometry_field='geom',
                            autodetect_key_field=True)
        fs = ds.featureset()
        for id in range(1, 5):
            eq_(fs.next().id(), id)

        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), u'gid')
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_empty_geom():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test7',
                            geometry_field='geom')
        fs = ds.featureset()
        eq_(fs.next()['gid'], 1)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Collection)

    def create_ds():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,
                            table='test',
                            max_size=20,
                            geometry_field='geom')
        fs = list(ds.all_features())
        eq_(len(fs), 8)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Collection)

    def test_threaded_create(NUM_THREADS=100):
        # run one to start before thread loop
        # to ensure that a throw stops the test
        # from running all threads
        create_ds()
        runs = 0
        for i in range(NUM_THREADS):
            t = threading.Thread(target=create_ds)
            t.start()
            t.join()
            runs += 1
        eq_(runs, NUM_THREADS)

    def create_ds_and_error():
        try:
            ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,
                                table='asdfasdfasdfasdfasdf',
                                max_size=20)
            ds.all_features()
        except Exception as e:
            eq_('in executeQuery' in str(e), True)

    def test_threaded_create2(NUM_THREADS=10):
        for i in range(NUM_THREADS):
            t = threading.Thread(target=create_ds_and_error)
            t.start()
            t.join()

    def test_that_64bit_int_fields_work():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,
                            table='test8',
                            geometry_field='geom')
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['gid', 'int_field'])
        eq_(ds.field_types(), ['int', 'int'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat.id(), 1)
        eq_(feat['gid'], 1)
        eq_(feat['int_field'], 2147483648)
        feat = fs.next()
        eq_(feat.id(), 2)
        eq_(feat['gid'], 2)
        eq_(feat['int_field'], 922337203685477580)

        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_persist_connection_off():
        # NOTE: max_size should be equal or greater than
        #       the pool size. There's currently no API to
        #       check nor set that size, but the current
        #       default is 20, so we use that value. See
        #       http://github.com/mapnik/mapnik/issues/863
        max_size = 20
        for i in range(0, max_size + 1):
            ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,
                                max_size=1,  # unused
                                persist_connection=False,
                                table='(select ST_MakePoint(0,0) as g, pg_backend_pid() as p, 1 as v) as w',
                                geometry_field='g')
            fs = ds.featureset()
            eq_(fs.next()['v'], 1)

            meta = ds.describe()
            eq_(meta['srid'], -1)
            eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    def test_null_comparision():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test9',
                            geometry_field='geom')
        fs = ds.featureset()
        feat = fs.next()

        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

        eq_(feat['gid'], 1)
        eq_(feat['name'], 'name')
        eq_(mapnik.Expression("[name] = 'name'").evaluate(feat), True)
        eq_(mapnik.Expression("[name] = ''").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = null").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = true").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = false").evaluate(feat), False)
        eq_(mapnik.Expression("[name] != 'name'").evaluate(feat), False)
        eq_(mapnik.Expression("[name] != ''").evaluate(feat), True)
        eq_(mapnik.Expression("[name] != null").evaluate(feat), True)
        eq_(mapnik.Expression("[name] != true").evaluate(feat), True)
        eq_(mapnik.Expression("[name] != false").evaluate(feat), True)

        feat = fs.next()
        eq_(feat['gid'], 2)
        eq_(feat['name'], '')
        eq_(mapnik.Expression("[name] = 'name'").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = ''").evaluate(feat), True)
        eq_(mapnik.Expression("[name] = null").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = true").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = false").evaluate(feat), False)
        eq_(mapnik.Expression("[name] != 'name'").evaluate(feat), True)
        eq_(mapnik.Expression("[name] != ''").evaluate(feat), False)
        eq_(mapnik.Expression("[name] != null").evaluate(feat), True)
        eq_(mapnik.Expression("[name] != true").evaluate(feat), True)
        eq_(mapnik.Expression("[name] != false").evaluate(feat), True)

        feat = fs.next()
        eq_(feat['gid'], 3)
        eq_(feat['name'], None)  # null
        eq_(mapnik.Expression("[name] = 'name'").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = ''").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = null").evaluate(feat), True)
        eq_(mapnik.Expression("[name] = true").evaluate(feat), False)
        eq_(mapnik.Expression("[name] = false").evaluate(feat), False)
        eq_(mapnik.Expression("[name] != 'name'").evaluate(feat), True)
        # https://github.com/mapnik/mapnik/issues/1859
        eq_(mapnik.Expression("[name] != ''").evaluate(feat), False)
        eq_(mapnik.Expression("[name] != null").evaluate(feat), False)
        eq_(mapnik.Expression("[name] != true").evaluate(feat), True)
        eq_(mapnik.Expression("[name] != false").evaluate(feat), True)

    def test_null_comparision2():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='test10',
                            geometry_field='geom')
        fs = ds.featureset()
        feat = fs.next()

        meta = ds.describe()
        eq_(meta['srid'], -1)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

        eq_(feat['gid'], 1)
        eq_(feat['bool_field'], True)
        eq_(mapnik.Expression("[bool_field] = 'name'").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = ''").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = null").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = true").evaluate(feat), True)
        eq_(mapnik.Expression("[bool_field] = false").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] != 'name'").evaluate(feat), True)
        eq_(mapnik.Expression("[bool_field] != ''").evaluate(
            feat), True)  # in 2.1.x used to be False
        eq_(mapnik.Expression("[bool_field] != null").evaluate(
            feat), True)  # in 2.1.x used to be False
        eq_(mapnik.Expression("[bool_field] != true").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] != false").evaluate(feat), True)

        feat = fs.next()
        eq_(feat['gid'], 2)
        eq_(feat['bool_field'], False)
        eq_(mapnik.Expression("[bool_field] = 'name'").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = ''").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = null").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = true").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = false").evaluate(feat), True)
        eq_(mapnik.Expression("[bool_field] != 'name'").evaluate(feat), True)
        eq_(mapnik.Expression("[bool_field] != ''").evaluate(feat), True)
        eq_(mapnik.Expression("[bool_field] != null").evaluate(
            feat), True)  # in 2.1.x used to be False
        eq_(mapnik.Expression("[bool_field] != true").evaluate(feat), True)
        eq_(mapnik.Expression("[bool_field] != false").evaluate(feat), False)

        feat = fs.next()
        eq_(feat['gid'], 3)
        eq_(feat['bool_field'], None)  # null
        eq_(mapnik.Expression("[bool_field] = 'name'").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = ''").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = null").evaluate(feat), True)
        eq_(mapnik.Expression("[bool_field] = true").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] = false").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] != 'name'").evaluate(
            feat), True)  # in 2.1.x used to be False
        # https://github.com/mapnik/mapnik/issues/1859
        eq_(mapnik.Expression("[bool_field] != ''").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] != null").evaluate(feat), False)
        eq_(mapnik.Expression("[bool_field] != true").evaluate(
            feat), True)  # in 2.1.x used to be False
        eq_(mapnik.Expression("[bool_field] != false").evaluate(
            feat), True)  # in 2.1.x used to be False

    # https://github.com/mapnik/mapnik/issues/1816
    def test_exception_message_reporting():
        try:
            mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='doesnotexist')
        except Exception as e:
            eq_(str(e) != 'unidentifiable C++ exception', True)

    def test_null_id_field():
        opts = {'type': 'postgis',
                'dbname': MAPNIK_TEST_DBNAME,
                'geometry_field': 'geom',
                'table': "(select null::bigint as osm_id, GeomFromEWKT('SRID=4326;POINT(0 0)') as geom) as tmp"}
        ds = mapnik.Datasource(**opts)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat.id(), long(1))
        eq_(feat['osm_id'], None)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

    @raises(StopIteration)
    def test_null_key_field():
        opts = {'type': 'postgis',
                "key_field": 'osm_id',
                'dbname': MAPNIK_TEST_DBNAME,
                'geometry_field': 'geom',
                'table': "(select null::bigint as osm_id, GeomFromEWKT('SRID=4326;POINT(0 0)') as geom) as tmp"}
        ds = mapnik.Datasource(**opts)
        fs = ds.featureset()
        # should throw since key_field is null: StopIteration: No more
        # features.
        fs.next()

    def test_psql_error_should_not_break_connection_pool():
        # Bad request, will trigger an error when returning result
        ds_bad = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table="""(SELECT geom as geom,label::int from test11) as failure_table""",
                                max_async_connection=5, geometry_field='geom', srid=4326)

        # Good request
        ds_good = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table="test",
                                 max_async_connection=5, geometry_field='geom', srid=4326)

        # This will/should trigger a PSQL error
        failed = False
        try:
            fs = ds_bad.featureset()
            count = sum(1 for f in fs)
        except RuntimeError as e:
            assert 'invalid input syntax for integer' in str(e)
            failed = True

        eq_(failed, True)

        # Should be ok
        fs = ds_good.featureset()
        count = sum(1 for f in fs)
        eq_(count, 8)

    def test_psql_error_should_give_back_connections_opened_for_lower_layers_to_the_pool():
        map1 = mapnik.Map(600, 300)
        s = mapnik.Style()
        r = mapnik.Rule()
        r.symbols.append(mapnik.PolygonSymbolizer())
        s.rules.append(r)
        map1.append_style('style', s)

        # This layer will fail after a while
        buggy_s = mapnik.Style()
        buggy_r = mapnik.Rule()
        buggy_r.symbols.append(mapnik.PolygonSymbolizer())
        buggy_r.filter = mapnik.Filter("[fips] = 'FR'")
        buggy_s.rules.append(buggy_r)
        map1.append_style('style for buggy layer', buggy_s)
        buggy_layer = mapnik.Layer('this layer is buggy at runtime')
        # We ensure the query wille be long enough
        buggy_layer.datasource = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='(SELECT geom as geom, pg_sleep(0.1), fips::int from world_merc) as failure_tabl',
                                                max_async_connection=2, max_size=2, asynchronous_request=True, geometry_field='geom')
        buggy_layer.styles.append('style for buggy layer')

        # The query for this layer will be sent, then the previous layer will
        # raise an exception before results are read
        forced_canceled_layer = mapnik.Layer(
            'this layer will be canceled when an exception stops map rendering')
        forced_canceled_layer.datasource = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='world_merc',
                                                          max_async_connection=2, max_size=2, asynchronous_request=True, geometry_field='geom')
        forced_canceled_layer.styles.append('style')

        map1.layers.append(buggy_layer)
        map1.layers.append(forced_canceled_layer)
        map1.zoom_all()
        map2 = mapnik.Map(600, 300)
        map2.background = mapnik.Color('steelblue')
        s = mapnik.Style()
        r = mapnik.Rule()
        r.symbols.append(mapnik.LineSymbolizer())
        r.symbols.append(mapnik.LineSymbolizer())
        s.rules.append(r)
        map2.append_style('style', s)
        layer1 = mapnik.Layer('layer1')
        layer1.datasource = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='world_merc',
                                           max_async_connection=2, max_size=2, asynchronous_request=True, geometry_field='geom')
        layer1.styles.append('style')
        map2.layers.append(layer1)
        map2.zoom_all()

        # We expect this to trigger a PSQL error
        try:
            mapnik.render_to_file(
                map1, '/tmp/mapnik-postgis-test-map1.png', 'png')
            # Test must fail if error was not raised just above
            eq_(False, True)
        except RuntimeError as e:
            assert 'invalid input syntax for integer' in str(e)
            pass
        # This used to raise an exception before correction of issue 2042
        mapnik.render_to_file(map2, '/tmp/mapnik-postgis-test-map2.png', 'png')

    def test_handling_of_zm_dimensions():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,
                            table='(select gid,ST_CoordDim(geom) as dim,name,geom from test12) as tmp',
                            geometry_field='geom')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['gid', 'dim', 'name'])
        eq_(ds.field_types(), ['int', 'int', 'str'])
        fs = ds.featureset()

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), None)
        # Note: this is incorrect because we only check first couple geoms
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)

        # Point (2d)
        feat = fs.next()
        eq_(feat.id(), 1)
        eq_(feat['gid'], 1)
        eq_(feat['dim'], 2)
        eq_(feat['name'], 'Point')
        eq_(feat.geometry.to_wkt(), 'POINT(0 0)')

        # PointZ
        feat = fs.next()
        eq_(feat.id(), 2)
        eq_(feat['gid'], 2)
        eq_(feat['dim'], 3)
        eq_(feat['name'], 'PointZ')
        eq_(feat.geometry.to_wkt(), 'POINT(0 0)')

        # PointM
        feat = fs.next()
        eq_(feat.id(), 3)
        eq_(feat['gid'], 3)
        eq_(feat['dim'], 3)
        eq_(feat['name'], 'PointM')
        eq_(feat.geometry.to_wkt(), 'POINT(0 0)')

        # PointZM
        feat = fs.next()
        eq_(feat.id(), 4)
        eq_(feat['gid'], 4)
        eq_(feat['dim'], 4)
        eq_(feat['name'], 'PointZM')

        eq_(feat.geometry.to_wkt(), 'POINT(0 0)')
        # MultiPoint
        feat = fs.next()
        eq_(feat.id(), 5)
        eq_(feat['gid'], 5)
        eq_(feat['dim'], 2)
        eq_(feat['name'], 'MultiPoint')
        eq_(feat.geometry.to_wkt(), 'MULTIPOINT(0 0,1 1)')

        # MultiPointZ
        feat = fs.next()
        eq_(feat.id(), 6)
        eq_(feat['gid'], 6)
        eq_(feat['dim'], 3)
        eq_(feat['name'], 'MultiPointZ')
        eq_(feat.geometry.to_wkt(), 'MULTIPOINT(0 0,1 1)')

        # MultiPointM
        feat = fs.next()
        eq_(feat.id(), 7)
        eq_(feat['gid'], 7)
        eq_(feat['dim'], 3)
        eq_(feat['name'], 'MultiPointM')
        eq_(feat.geometry.to_wkt(), 'MULTIPOINT(0 0,1 1)')

        # MultiPointZM
        feat = fs.next()
        eq_(feat.id(), 8)
        eq_(feat['gid'], 8)
        eq_(feat['dim'], 4)
        eq_(feat['name'], 'MultiPointZM')
        eq_(feat.geometry.to_wkt(), 'MULTIPOINT(0 0,1 1)')

        # LineString
        feat = fs.next()
        eq_(feat.id(), 9)
        eq_(feat['gid'], 9)
        eq_(feat['dim'], 2)
        eq_(feat['name'], 'LineString')
        eq_(feat.geometry.to_wkt(), 'LINESTRING(0 0,1 1)')

        # LineStringZ
        feat = fs.next()
        eq_(feat.id(), 10)
        eq_(feat['gid'], 10)
        eq_(feat['dim'], 3)
        eq_(feat['name'], 'LineStringZ')
        eq_(feat.geometry.to_wkt(), 'LINESTRING(0 0,1 1)')

        # LineStringM
        feat = fs.next()
        eq_(feat.id(), 11)
        eq_(feat['gid'], 11)
        eq_(feat['dim'], 3)
        eq_(feat['name'], 'LineStringM')
        eq_(feat.geometry.to_wkt(), 'LINESTRING(0 0,1 1)')

        # LineStringZM
        feat = fs.next()
        eq_(feat.id(), 12)
        eq_(feat['gid'], 12)
        eq_(feat['dim'], 4)
        eq_(feat['name'], 'LineStringZM')
        eq_(feat.geometry.to_wkt(), 'LINESTRING(0 0,1 1)')

        # Polygon
        feat = fs.next()
        eq_(feat.id(), 13)
        eq_(feat['gid'], 13)
        eq_(feat['name'], 'Polygon')
        eq_(feat.geometry.to_wkt(), 'POLYGON((0 0,1 1,2 2,0 0))')

        # PolygonZ
        feat = fs.next()
        eq_(feat.id(), 14)
        eq_(feat['gid'], 14)
        eq_(feat['name'], 'PolygonZ')
        eq_(feat.geometry.to_wkt(), 'POLYGON((0 0,1 1,2 2,0 0))')

        # PolygonM
        feat = fs.next()
        eq_(feat.id(), 15)
        eq_(feat['gid'], 15)
        eq_(feat['name'], 'PolygonM')
        eq_(feat.geometry.to_wkt(), 'POLYGON((0 0,1 1,2 2,0 0))')

        # PolygonZM
        feat = fs.next()
        eq_(feat.id(), 16)
        eq_(feat['gid'], 16)
        eq_(feat['name'], 'PolygonZM')
        eq_(feat.geometry.to_wkt(), 'POLYGON((0 0,1 1,2 2,0 0))')

        # MultiLineString
        feat = fs.next()
        eq_(feat.id(), 17)
        eq_(feat['gid'], 17)
        eq_(feat['name'], 'MultiLineString')
        eq_(feat.geometry.to_wkt(), 'MULTILINESTRING((0 0,1 1),(2 2,3 3))')

        # MultiLineStringZ
        feat = fs.next()
        eq_(feat.id(), 18)
        eq_(feat['gid'], 18)
        eq_(feat['name'], 'MultiLineStringZ')
        eq_(feat.geometry.to_wkt(), 'MULTILINESTRING((0 0,1 1),(2 2,3 3))')

        # MultiLineStringM
        feat = fs.next()
        eq_(feat.id(), 19)
        eq_(feat['gid'], 19)
        eq_(feat['name'], 'MultiLineStringM')
        eq_(feat.geometry.to_wkt(), 'MULTILINESTRING((0 0,1 1),(2 2,3 3))')

        # MultiLineStringZM
        feat = fs.next()
        eq_(feat.id(), 20)
        eq_(feat['gid'], 20)
        eq_(feat['name'], 'MultiLineStringZM')
        eq_(feat.geometry.to_wkt(), 'MULTILINESTRING((0 0,1 1),(2 2,3 3))')

        # MultiPolygon
        feat = fs.next()
        eq_(feat.id(), 21)
        eq_(feat['gid'], 21)
        eq_(feat['name'], 'MultiPolygon')
        eq_(feat.geometry.to_wkt(),
            'MULTIPOLYGON(((0 0,1 1,2 2,0 0)),((0 0,1 1,2 2,0 0)))')

        # MultiPolygonZ
        feat = fs.next()
        eq_(feat.id(), 22)
        eq_(feat['gid'], 22)
        eq_(feat['name'], 'MultiPolygonZ')
        eq_(feat.geometry.to_wkt(),
            'MULTIPOLYGON(((0 0,1 1,2 2,0 0)),((0 0,1 1,2 2,0 0)))')

        # MultiPolygonM
        feat = fs.next()
        eq_(feat.id(), 23)
        eq_(feat['gid'], 23)
        eq_(feat['name'], 'MultiPolygonM')
        eq_(feat.geometry.to_wkt(),
            'MULTIPOLYGON(((0 0,1 1,2 2,0 0)),((0 0,1 1,2 2,0 0)))')

        # MultiPolygonZM
        feat = fs.next()
        eq_(feat.id(), 24)
        eq_(feat['gid'], 24)
        eq_(feat['name'], 'MultiPolygonZM')
        eq_(feat.geometry.to_wkt(),
            'MULTIPOLYGON(((0 0,1 1,2 2,0 0)),((0 0,1 1,2 2,0 0)))')

    def test_handling_of_discarded_key_field():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME,
                            table='(select * from test12) as tmp',
                            key_field='gid',
                            key_field_as_attribute=False)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['name'],'Point')

    def test_variable_in_subquery1():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''
           (select * from test where @zoom = 30 ) as tmp''',
                            geometry_field='geom', srid=4326,
                            autodetect_key_field=True)
        fs = ds.featureset(variables={'zoom': 30})
        for id in range(1, 5):
            eq_(fs.next().id(), id)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta.get('key_field'), "gid")
        eq_(meta['geometry_type'], None)

    # currently needs manual `geometry_table` passed
    # to avoid misparse of `geometry_table`
    # in the future ideally this would not need manual  `geometry_table`
    # https://github.com/mapnik/mapnik/issues/2718
    # currently `bogus` would be picked automatically for geometry_table
    def test_broken_parsing_of_comments():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''
             (select * FROM test) AS data
             -- select this from bogus''',
                            geometry_table='test')
        fs = ds.featureset()
        for id in range(1, 5):
            eq_(fs.next().id(), id)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Collection)

    # same
    # to avoid misparse of `geometry_table`
    # in the future ideally this would not need manual  `geometry_table`
    # https://github.com/mapnik/mapnik/issues/2718
    # currently nothing would be picked automatically for geometry_table
    def test_broken_parsing_of_comments():
        ds = mapnik.PostGIS(dbname=MAPNIK_TEST_DBNAME, table='''
             (select * FROM test) AS data
             -- select this from bogus.''',
                            geometry_table='test')
        fs = ds.featureset()
        for id in range(1, 5):
            eq_(fs.next().id(), id)

        meta = ds.describe()
        eq_(meta['srid'], 4326)
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Collection)

    atexit.register(postgis_takedown)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import assert_almost_equal, eq_, raises

import mapnik

from .utilities import execution_path, run_all

try:
    import json
except ImportError:
    import simplejson as json


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if 'ogr' in mapnik.DatasourceCache.plugin_names():

    # Shapefile initialization
    def test_shapefile_init():
        ds = mapnik.Ogr(file='../data/shp/boundaries.shp', layer_by_index=0)
        e = ds.envelope()
        assert_almost_equal(e.minx, -11121.6896651, places=7)
        assert_almost_equal(e.miny, -724724.216526, places=6)
        assert_almost_equal(e.maxx, 2463000.67866, places=5)
        assert_almost_equal(e.maxy, 1649661.267, places=3)
        meta = ds.describe()
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Polygon)
        eq_('+proj=lcc' in meta['proj4'], True)

    # Shapefile properties
    def test_shapefile_properties():
        ds = mapnik.Ogr(file='../data/shp/boundaries.shp', layer_by_index=0)
        f = list(ds.features_at_point(ds.envelope().center(), 0.001))[0]
        eq_(ds.geometry_type(), mapnik.DataGeometryType.Polygon)

        eq_(f['CGNS_FID'], u'6f733341ba2011d892e2080020a0f4c9')
        eq_(f['COUNTRY'], u'CAN')
        eq_(f['F_CODE'], u'FA001')
        eq_(f['NAME_EN'], u'Quebec')
        eq_(f['Shape_Area'], 1512185733150.0)
        eq_(f['Shape_Leng'], 19218883.724300001)
        meta = ds.describe()
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Polygon)
        # NOTE: encoding is latin1 but gdal >= 1.9 should now expose utf8 encoded features
        # See SHAPE_ENCODING for overriding: http://gdal.org/ogr/drv_shapefile.html
        # Failure for the NOM_FR field is expected for older gdal
        #eq_(f['NOM_FR'], u'Qu\xe9bec')
        #eq_(f['NOM_FR'], u'Québec')

    @raises(RuntimeError)
    def test_that_nonexistant_query_field_throws(**kwargs):
        ds = mapnik.Ogr(file='../data/shp/world_merc.shp', layer_by_index=0)
        eq_(len(ds.fields()), 11)
        eq_(ds.fields(), ['FIPS', 'ISO2', 'ISO3', 'UN', 'NAME',
                          'AREA', 'POP2005', 'REGION', 'SUBREGION', 'LON', 'LAT'])
        eq_(ds.field_types(),
            ['str',
             'str',
             'str',
             'int',
             'str',
             'int',
             'int',
             'int',
             'int',
             'float',
             'float'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        # also add an invalid one, triggering throw
        query.add_property_name('bogus')
        ds.features(query)

    # disabled because OGR prints an annoying error: ERROR 1: Invalid Point object. Missing 'coordinates' member.
    # def test_handling_of_null_features():
    #    ds = mapnik.Ogr(file='../data/json/null_feature.geojson',layer_by_index=0)
    #    fs = ds.all_features()
    #    eq_(len(fs),1)

    # OGR plugin extent parameter
    def test_ogr_extent_parameter():
        ds = mapnik.Ogr(
            file='../data/shp/world_merc.shp',
            layer_by_index=0,
            extent='-1,-1,1,1')
        e = ds.envelope()
        eq_(e.minx, -1)
        eq_(e.miny, -1)
        eq_(e.maxx, 1)
        eq_(e.maxy, 1)
        meta = ds.describe()
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Polygon)
        eq_('+proj=merc' in meta['proj4'], True)

    def test_ogr_reading_gpx_waypoint():
        ds = mapnik.Ogr(file='../data/gpx/empty.gpx', layer='waypoints')
        e = ds.envelope()
        eq_(e.minx, -122)
        eq_(e.miny, 48)
        eq_(e.maxx, -122)
        eq_(e.maxy, 48)
        meta = ds.describe()
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)
        eq_('+proj=longlat' in meta['proj4'], True)

    def test_ogr_empty_data_should_not_throw():
        default_logging_severity = mapnik.logger.get_severity()
        mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
        # use logger to silence expected warnings
        for layer in ['routes', 'tracks', 'route_points', 'track_points']:
            ds = mapnik.Ogr(file='../data/gpx/empty.gpx', layer=layer)
            e = ds.envelope()
            eq_(e.minx, 0)
            eq_(e.miny, 0)
            eq_(e.maxx, 0)
            eq_(e.maxy, 0)
        mapnik.logger.set_severity(default_logging_severity)
        meta = ds.describe()
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)
        eq_('+proj=longlat' in meta['proj4'], True)

    # disabled because OGR prints an annoying error: ERROR 1: Invalid Point object. Missing 'coordinates' member.
    # def test_handling_of_null_features():
    #    ds = mapnik.Ogr(file='../data/json/null_feature.geojson',layer_by_index=0)
    #    fs = ds.all_features()
    #    eq_(len(fs),1)

    def test_geometry_type():
        ds = mapnik.Ogr(file='../data/csv/wkt.csv', layer_by_index=0)
        e = ds.envelope()
        assert_almost_equal(e.minx, 1.0, places=1)
        assert_almost_equal(e.miny, 1.0, places=1)
        assert_almost_equal(e.maxx, 45.0, places=1)
        assert_almost_equal(e.maxy, 45.0, places=1)
        meta = ds.describe()
        eq_(meta['geometry_type'], mapnik.DataGeometryType.Point)
        #eq_('+proj=longlat' in meta['proj4'],True)
        fs = ds.featureset()
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual,
            {u'geometry': {u'type': u'Point',
                           u'coordinates': [30,
                                            10]},
             u'type': u'Feature',
             u'id': 2,
             u'properties': {u'type': u'point',
                             u'WKT': u'           POINT (30 10)'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual,
            {u'geometry': {u'type': u'LineString',
                           u'coordinates': [[30,
                                             10],
                                            [10,
                                             30],
                                            [40,
                                             40]]},
                u'type': u'Feature',
                u'id': 3,
                u'properties': {u'type': u'linestring',
                                u'WKT': u'      LINESTRING (30 10, 10 30, 40 40)'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual,
            {u'geometry': {u'type': u'Polygon',
                           u'coordinates': [[[30,
                                              10],
                                             [40,
                                              40],
                                             [20,
                                              40],
                                             [10,
                                              20],
                                             [30,
                                              10]]]},
                u'type': u'Feature',
                u'id': 4,
                u'properties': {u'type': u'polygon',
                                u'WKT': u'         POLYGON ((30 10, 10 20, 20 40, 40 40, 30 10))'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(
            actual, {
                u'geometry': {
                    u'type': u'Polygon', u'coordinates': [
                        [
                            [
                                35, 10], [
                                45, 45], [
                                15, 40], [
                                    10, 20], [
                                        35, 10]], [
                                            [
                                                20, 30], [
                                                    35, 35], [
                                                        30, 20], [
                                                            20, 30]]]}, u'type': u'Feature', u'id': 5, u'properties': {
                                                                u'type': u'polygon', u'WKT': u'         POLYGON ((35 10, 10 20, 15 40, 45 45, 35 10),(20 30, 35 35, 30 20, 20 30))'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual,
            {u'geometry': {u'type': u'MultiPoint',
                           u'coordinates': [[10,
                                             40],
                                            [40,
                                             30],
                                            [20,
                                             20],
                                            [30,
                                             10]]},
                u'type': u'Feature',
                u'id': 6,
                u'properties': {u'type': u'multipoint',
                                u'WKT': u'      MULTIPOINT ((10 40), (40 30), (20 20), (30 10))'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual,
            {u'geometry': {u'type': u'MultiLineString',
                           u'coordinates': [[[10,
                                              10],
                                             [20,
                                              20],
                                             [10,
                                              40]],
                                            [[40,
                                              40],
                                             [30,
                                                30],
                                             [40,
                                                20],
                                             [30,
                                                10]]]},
                u'type': u'Feature',
                u'id': 7,
                u'properties': {u'type': u'multilinestring',
                                u'WKT': u' MULTILINESTRING ((10 10, 20 20, 10 40),(40 40, 30 30, 40 20, 30 10))'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual,
            {u'geometry': {u'type': u'MultiPolygon',
                           u'coordinates': [[[[30,
                                               20],
                                              [45,
                                               40],
                                              [10,
                                               40],
                                              [30,
                                               20]]],
                                            [[[15,
                                               5],
                                              [40,
                                                10],
                                                [10,
                                                 20],
                                                [5,
                                                 10],
                                                [15,
                                                 5]]]]},
                u'type': u'Feature',
                u'id': 8,
                u'properties': {u'type': u'multipolygon',
                                u'WKT': u'    MULTIPOLYGON (((30 20, 10 40, 45 40, 30 20)),((15 5, 40 10, 10 20, 5 10, 15 5)))'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual, {u'geometry': {u'type': u'MultiPolygon', u'coordinates': [[[[40, 40], [20, 45], [45, 30], [40, 40]]], [[[20, 35], [10, 30], [10, 10], [30, 5], [45, 20], [20, 35]], [[30, 20], [20, 15], [20, 25], [
            30, 20]]]]}, u'type': u'Feature', u'id': 9, u'properties': {u'type': u'multipolygon', u'WKT': u'    MULTIPOLYGON (((40 40, 20 45, 45 30, 40 40)),((20 35, 45 20, 30 5, 10 10, 10 30, 20 35),(30 20, 20 25, 20 15, 30 20)))'}})
        feat = fs.next()
        actual = json.loads(feat.to_geojson())
        eq_(actual,
            {u'geometry': {u'type': u'GeometryCollection',
                           u'geometries': [{u'type': u'Polygon',
                                            u'coordinates': [[[1,
                                                               1],
                                                              [2,
                                                               1],
                                                              [2,
                                                               2],
                                                              [1,
                                                               2],
                                                              [1,
                                                               1]]]},
                                           {u'type': u'Point',
                                            u'coordinates': [2,
                                                             3]},
                                           {u'type': u'LineString',
                                            u'coordinates': [[2,
                                                              3],
                                                             [3,
                                                              4]]}]},
                u'type': u'Feature',
                u'id': 10,
                u'properties': {u'type': u'collection',
                                u'WKT': u'      GEOMETRYCOLLECTION(POLYGON((1 1,2 1,2 2,1 2,1 1)),POINT(2 3),LINESTRING(2 3,3 4))'}})

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))
